// python wrapper for vtkBoostDividedEdgeBundling
//
#define VTK_WRAPPING_CXX
#define VTK_STREAMS_FWD_ONLY
#include "vtkPythonArgs.h"
#include "vtkPythonOverload.h"
#include "vtkConfigure.h"
#include <cstddef>
#include <sstream>
#include "vtkVariant.h"
#include "vtkIndent.h"
#include "vtkBoostDividedEdgeBundling.h"

extern "C" { VTK_ABI_EXPORT void PyVTKAddFile_vtkBoostDividedEdgeBundling(PyObject *); }
extern "C" { VTK_ABI_EXPORT PyObject *PyvtkBoostDividedEdgeBundling_ClassNew(); }

#ifndef DECLARED_PyvtkDirectedGraphAlgorithm_ClassNew
extern "C" { PyObject *PyvtkDirectedGraphAlgorithm_ClassNew(); }
#define DECLARED_PyvtkDirectedGraphAlgorithm_ClassNew
#endif

static const char *PyvtkBoostDividedEdgeBundling_Doc =
  "vtkBoostDividedEdgeBundling - layout graph edges in directed edge\nbundles\n\n"
  "Superclass: vtkDirectedGraphAlgorithm\n\n"
  "Uses the technique by Selassie, Heller, and Heer to route graph edges\n"
  "into directed bundles, with \"lanes\" for bundled edges moving in each\n"
  "direction. This technique works best for networks whose vertices have\n"
  "been positioned already (geospatial graphs, for example). Note that\n"
  "this scales to a few thousand edges in a reasonable period of time\n"
  "(~1 minute). The time complexity comes mainly from the doubling of\n"
  "edge control points each cycle and the complex set of forces between\n"
  "many pairs of edge points.\n\n"
  "The algorithm depends on the Boost graph library for its\n"
  "implementation of all-pairs shortest paths, needed here for\n"
  "determining connectivity compatibility.\n\n"
  "@par Thanks: This algorithm was developed in the paper:\n"
  "  David Selassie, Brandon Heller, Jeffrey Heer. Divided Edge Bundling\n"
  "for Directional\n"
  "  Network Data. Proceedings of IEEE InfoVis 2011.\n\n";


static PyObject *
PyvtkBoostDividedEdgeBundling_IsTypeOf(PyObject *, PyObject *args)
{
  vtkPythonArgs ap(args, "IsTypeOf");

  char *temp0 = nullptr;
  PyObject *result = nullptr;

  if (ap.CheckArgCount(1) &&
      ap.GetValue(temp0))
  {
    int tempr = vtkBoostDividedEdgeBundling::IsTypeOf(temp0);

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildValue(tempr);
    }
  }

  return result;
}


static PyObject *
PyvtkBoostDividedEdgeBundling_IsA(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "IsA");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkBoostDividedEdgeBundling *op = static_cast<vtkBoostDividedEdgeBundling *>(vp);

  char *temp0 = nullptr;
  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(1) &&
      ap.GetValue(temp0))
  {
    int tempr = (ap.IsBound() ?
      op->IsA(temp0) :
      op->vtkBoostDividedEdgeBundling::IsA(temp0));

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildValue(tempr);
    }
  }

  return result;
}


static PyObject *
PyvtkBoostDividedEdgeBundling_SafeDownCast(PyObject *, PyObject *args)
{
  vtkPythonArgs ap(args, "SafeDownCast");

  vtkObjectBase *temp0 = nullptr;
  PyObject *result = nullptr;

  if (ap.CheckArgCount(1) &&
      ap.GetVTKObject(temp0, "vtkObjectBase"))
  {
    vtkBoostDividedEdgeBundling *tempr = vtkBoostDividedEdgeBundling::SafeDownCast(temp0);

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildVTKObject(tempr);
    }
  }

  return result;
}


static PyObject *
PyvtkBoostDividedEdgeBundling_NewInstance(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "NewInstance");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkBoostDividedEdgeBundling *op = static_cast<vtkBoostDividedEdgeBundling *>(vp);

  PyObject *result = nullptr;

  if (op && ap.CheckArgCount(0))
  {
    vtkBoostDividedEdgeBundling *tempr = (ap.IsBound() ?
      op->NewInstance() :
      op->vtkBoostDividedEdgeBundling::NewInstance());

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildVTKObject(tempr);
      if (result && PyVTKObject_Check(result))
      {
        PyVTKObject_GetObject(result)->UnRegister(0);
        PyVTKObject_SetFlag(result, VTK_PYTHON_IGNORE_UNREGISTER, 1);
      }
    }
  }

  return result;
}

static PyMethodDef PyvtkBoostDividedEdgeBundling_Methods[] = {
  {"IsTypeOf", PyvtkBoostDividedEdgeBundling_IsTypeOf, METH_VARARGS,
   "V.IsTypeOf(string) -> int\nC++: static vtkTypeBool IsTypeOf(const char *type)\n\nReturn 1 if this class type is the same type of (or a subclass\nof) the named class. Returns 0 otherwise. This method works in\ncombination with vtkTypeMacro found in vtkSetGet.h.\n"},
  {"IsA", PyvtkBoostDividedEdgeBundling_IsA, METH_VARARGS,
   "V.IsA(string) -> int\nC++: vtkTypeBool IsA(const char *type) override;\n\nReturn 1 if this class is the same type of (or a subclass of) the\nnamed class. Returns 0 otherwise. This method works in\ncombination with vtkTypeMacro found in vtkSetGet.h.\n"},
  {"SafeDownCast", PyvtkBoostDividedEdgeBundling_SafeDownCast, METH_VARARGS,
   "V.SafeDownCast(vtkObjectBase) -> vtkBoostDividedEdgeBundling\nC++: static vtkBoostDividedEdgeBundling *SafeDownCast(\n    vtkObjectBase *o)\n\n"},
  {"NewInstance", PyvtkBoostDividedEdgeBundling_NewInstance, METH_VARARGS,
   "V.NewInstance() -> vtkBoostDividedEdgeBundling\nC++: vtkBoostDividedEdgeBundling *NewInstance()\n\n"},
  {nullptr, nullptr, 0, nullptr}
};

static PyTypeObject PyvtkBoostDividedEdgeBundling_Type = {
  PyVarObject_HEAD_INIT(&PyType_Type, 0)
  "vtkInfovisBoostGraphAlgorithmsPython.vtkBoostDividedEdgeBundling", // tp_name
  sizeof(PyVTKObject), // tp_basicsize
  0, // tp_itemsize
  PyVTKObject_Delete, // tp_dealloc
  nullptr, // tp_print
  nullptr, // tp_getattr
  nullptr, // tp_setattr
  nullptr, // tp_compare
  PyVTKObject_Repr, // tp_repr
  nullptr, // tp_as_number
  nullptr, // tp_as_sequence
  nullptr, // tp_as_mapping
  nullptr, // tp_hash
  nullptr, // tp_call
  PyVTKObject_String, // tp_str
  PyObject_GenericGetAttr, // tp_getattro
  PyObject_GenericSetAttr, // tp_setattro
  &PyVTKObject_AsBuffer, // tp_as_buffer
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_BASETYPE, // tp_flags
  PyvtkBoostDividedEdgeBundling_Doc, // tp_doc
  PyVTKObject_Traverse, // tp_traverse
  nullptr, // tp_clear
  nullptr, // tp_richcompare
  offsetof(PyVTKObject, vtk_weakreflist), // tp_weaklistoffset
  nullptr, // tp_iter
  nullptr, // tp_iternext
  nullptr, // tp_methods
  nullptr, // tp_members
  PyVTKObject_GetSet, // tp_getset
  nullptr, // tp_base
  nullptr, // tp_dict
  nullptr, // tp_descr_get
  nullptr, // tp_descr_set
  offsetof(PyVTKObject, vtk_dict), // tp_dictoffset
  nullptr, // tp_init
  nullptr, // tp_alloc
  PyVTKObject_New, // tp_new
  PyObject_GC_Del, // tp_free
  nullptr, // tp_is_gc
  nullptr, // tp_bases
  nullptr, // tp_mro
  nullptr, // tp_cache
  nullptr, // tp_subclasses
  nullptr, // tp_weaklist
  VTK_WRAP_PYTHON_SUPPRESS_UNINITIALIZED
};

static vtkObjectBase *PyvtkBoostDividedEdgeBundling_StaticNew()
{
  return vtkBoostDividedEdgeBundling::New();
}

PyObject *PyvtkBoostDividedEdgeBundling_ClassNew()
{
  PyVTKClass_Add(
    &PyvtkBoostDividedEdgeBundling_Type, PyvtkBoostDividedEdgeBundling_Methods,
    "vtkBoostDividedEdgeBundling",
 &PyvtkBoostDividedEdgeBundling_StaticNew);

  PyTypeObject *pytype = &PyvtkBoostDividedEdgeBundling_Type;

  if ((pytype->tp_flags & Py_TPFLAGS_READY) != 0)
  {
    return (PyObject *)pytype;
  }

#if !defined(VTK_PY3K) && PY_VERSION_HEX >= 0x02060000
  pytype->tp_flags |= Py_TPFLAGS_HAVE_NEWBUFFER;
#endif

  pytype->tp_base = (PyTypeObject *)PyvtkDirectedGraphAlgorithm_ClassNew();

  PyType_Ready(pytype);
  return (PyObject *)pytype;
}

void PyVTKAddFile_vtkBoostDividedEdgeBundling(
  PyObject *dict)
{
  PyObject *o;
  o = PyvtkBoostDividedEdgeBundling_ClassNew();

  if (o && PyDict_SetItemString(dict, "vtkBoostDividedEdgeBundling", o) != 0)
  {
    Py_DECREF(o);
  }

}

